// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.aws.subscriber;

import com.amazonaws.services.kms.AWSKMS;
import com.amazonaws.services.sns.AmazonSNS;
import com.amazonaws.services.sns.model.*;
import org.opengroup.osdu.core.aws.sns.AmazonSNSConfig;
import org.opengroup.osdu.core.aws.ssm.ParameterStorePropertySource;
import org.opengroup.osdu.core.aws.ssm.SSMConfig;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.provider.aws.config.AwsServiceConfig;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import java.util.List;

@Component
public class SubscriptionHelper {




    private ParameterStorePropertySource ssm;

    @Inject
    private JaxRsDpsLog logger;




    @Inject
    private AwsServiceConfig serviceConfig;


    private AmazonSNS snsClient;

    @PostConstruct
    public void init(){

        snsClient = serviceConfig.getSnsClient();


    }

    public String  createPushSubscription(String topicArn,String pushEndpoint) {
        //There is no way in aws java sdk to look up sns topic arn by name.
        //We might want to create a separate table for Topic -- TopicArn
        // For now retrieving the single topic created by cloudformation. This might change in the future if
        //multiple topics are created using Topics APIS.
        //See Gitlab issue:https://community.opengroup.org/osdu/platform/system/register/-/issues/14


        try {
            String env = System.getProperty("ENVIRONMENT", System.getenv("ENVIRONMENT"));
            String pushUrlprotocol="https";
            if(env.equalsIgnoreCase("LOCAL")) { // need http for local testing..
                pushUrlprotocol="http";
            }
            SubscribeRequest subscribeRequest = new SubscribeRequest(topicArn, pushUrlprotocol, pushEndpoint);
            subscribeRequest.setReturnSubscriptionArn(true);
            SubscribeResult subscriptionResult = snsClient.subscribe(subscribeRequest);
            String subscriptionArn = subscriptionResult.getSubscriptionArn();
            return subscriptionArn;
        }
        catch(Exception e){
           logger.error("Create subscription failed for topic name"+topicArn);
            throw e;
        }

    }

    public void  deletePushSubscription(String subscriptionArn) {

        UnsubscribeRequest unsubscribeRequest = new UnsubscribeRequest(subscriptionArn);
        try {
            UnsubscribeResult unSubscribeResult = snsClient.unsubscribe(unsubscribeRequest);
        }catch(AppException e)
        {
            if (e.getError().getCode() == 404) {
                logger.error(String.format("Subscription with ARN %s does not exist.", subscriptionArn));
                throw new AppException(404, "Not found", String.format("Subscription with ARN %s does not exist.", subscriptionArn));
            }else {
                logger.error(e.getMessage());
                throw new AppException(e.getError().getCode(), e.getError().getReason(), e.getMessage());
            }

        }


    }

    public boolean  doesSubscriptionExist(String subscriptionArn, String topicName) {

        //There is no way in aws java sdk to look up sns topic arn by name.
        //We might want to create a separate table for Topic -- TopicArn

        // For now retrieving the single topic created by cloudformation. This miht change in the future if
        //multiple topics are created using Topics APIS.
        //See Gitlab issue:https://community.opengroup.org/osdu/platform/system/register/-/issues/14
        SSMConfig ssmConfig = new SSMConfig();
        ParameterStorePropertySource ssm = ssmConfig.amazonSSM();
       // amazonSNSTopicArn = ssm.getProperty(awsServiceConfig.getSnsTopicArn()).toString();
        String amazonSNSTopicArn = topicName;
        ListSubscriptionsByTopicRequest listSubsRequest = new ListSubscriptionsByTopicRequest(amazonSNSTopicArn);
        //only returns the first 100, for the next lot pass nextToken

        ListSubscriptionsByTopicResult listSubsResult = snsClient.listSubscriptionsByTopic(listSubsRequest);
        List<Subscription> allSubsByTopic = listSubsResult.getSubscriptions();
        for(Subscription s: allSubsByTopic)
        {
            if(s.getSubscriptionArn().equals(subscriptionArn)){
                return true;
            }else{
                continue;
            }

        }

        return false;



    }


}
