// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.subscriber;

import com.azure.cosmos.models.CosmosQueryRequestOptions;
import com.azure.cosmos.models.SqlParameter;
import com.azure.cosmos.models.SqlQuerySpec;
import org.opengroup.osdu.azure.cosmosdb.CosmosStore;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.provider.azure.di.AzureBootstrapConfig;
import org.opengroup.osdu.register.provider.azure.di.CosmosContainerConfig;
import org.opengroup.osdu.register.provider.azure.subscriber.pubsub.IPubsubPartitionHelper;
import org.opengroup.osdu.register.provider.azure.util.CryptographyUtil;
import org.opengroup.osdu.register.subscriber.model.*;
import org.opengroup.osdu.register.utils.Constants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

@Component
public class CosmosSubscription {

    @Autowired
    private AzureBootstrapConfig azureBootstrapConfig;

    @Autowired
    private CosmosContainerConfig cosmosContainerConfig;

    @Autowired
    private CosmosStore cosmosStore;

    @Autowired
    private DpsHeaders dpsHeaders;

    @Autowired
    private JaxRsDpsLog logger;

    @Autowired
    private CryptographyUtil cryptographyUtil;

    @Autowired
    private IPubsubPartitionHelper partitionHelper;

    public Subscription createSubscription(Subscription input) throws AppException {
        SubscriptionDoc doc = new SubscriptionDoc(input, dpsHeaders.getPartitionId());
        doc.setTopic(partitionHelper.getFullyQualifiedTopicName(input.getTopic()));
        doc.setSecretValue(cryptographyUtil.encryptData(input.getSecret().toString()));
        doc.setKeyName(cryptographyUtil.getKeyName());
        doc.setKeyVersion(cryptographyUtil.getKeyVersion());
        cosmosStore.createItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), dpsHeaders.getPartitionId(), doc);
        logger.debug(String.format("Record successfully created for Subscription with id %s", input.getId()));
        return convertToSubscriptionClass(doc);
    }

    public void deleteSubscription(String subscriptionId) {
        cosmosStore.deleteItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(),
                cosmosContainerConfig.getSubscriptionContainerName(), subscriptionId, dpsHeaders.getPartitionId());
        logger.debug("Record deleted for subscription with ID: " + subscriptionId);
    }

    public Subscription getSubscription(String id) {
        SubscriptionDoc doc = cosmosStore.findItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), id, dpsHeaders.getPartitionId(), SubscriptionDoc.class)
                .orElseThrow(() -> {
                    logger.error(String.format("Subscriber with id %s does not exist.", id));
                    return new AppException(404, "Not found", String.format("Subscriber with id %s does not exist.", id));
                });
        logger.debug("Record fetched successfully for subscription with ID: " + id);
        return convertToSubscriptionClass(doc);
    }

    public List<Subscription> getAllSubscriptions() {

        SqlQuerySpec query = new SqlQuerySpec("SELECT * from c where c.dataPartitionId = @partitionId");

        List<SqlParameter> pars = query.getParameters();
        pars.add(new SqlParameter("@partitionId", dpsHeaders.getPartitionId()));

        CosmosQueryRequestOptions options = new CosmosQueryRequestOptions();

        return cosmosStore.queryItems(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), query, options, SubscriptionDoc.class).stream()
                .map(this::convertToSubscriptionClass)
                .collect(Collectors.toList());
    }

    public List<Subscription> querySubscription(String notificationId) {
        SqlQuerySpec query = new SqlQuerySpec("SELECT * from c where c.notificationId = @notificationId and c.dataPartitionId = @partitionId");

        List<SqlParameter> pars = query.getParameters();
        pars.add(new SqlParameter("@notificationId", notificationId));
        pars.add(new SqlParameter("@partitionId", dpsHeaders.getPartitionId()));

        CosmosQueryRequestOptions options = new CosmosQueryRequestOptions();

        return cosmosStore.queryItems(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), query, options, SubscriptionDoc.class).stream()
                .map(this::convertToSubscriptionClass)
                .collect(Collectors.toList());
    }

    public void updateSubscription(Subscription input) {
        SubscriptionDoc doc = new SubscriptionDoc(input, dpsHeaders.getPartitionId());
        doc.setTopic(partitionHelper.getFullyQualifiedTopicName(input.getTopic()));
        doc.setSecretValue(cryptographyUtil.encryptData(input.getSecret().toString()));
        doc.setKeyName(cryptographyUtil.getKeyName());
        doc.setKeyVersion(cryptographyUtil.getKeyVersion());
        cosmosStore.upsertItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), dpsHeaders.getPartitionId(), doc);
        logger.debug("Updating Record for subscription with ID: ", input.getId());
    }

    private Subscription convertToSubscriptionClass(SubscriptionDoc doc) {

        String secretValue = cryptographyUtil.decryptData(doc.getSecretValue());
        Secret secret;
        if (doc.getSecretType().equals(Constants.GSASecret)) {
            GsaSecret gsaSecret = new GsaSecret();
            String[] splitSecret = secretValue.split("`");
            gsaSecret.setValue(new GsaSecretValue(splitSecret[0], splitSecret[1]));
            secret = gsaSecret;
        } else {
            HmacSecret hmacSecret = new HmacSecret();
            hmacSecret.setValue(secretValue);
            secret = hmacSecret;
        }
        return new Subscription(doc.getId(), doc.getName(), doc.getDescription(),
                partitionHelper.getTopicAliasFromFullyQualifiedTopicName(doc.getTopic()), doc.getPushEndpoint(),
                doc.getCreatedBy(), doc.getCreatedOnEpoch(), doc.getNotificationId(), secret);
    }

    protected boolean updateSecret(Subscription subscription, Secret secret) {
        SubscriptionDoc doc = new SubscriptionDoc(subscription, dpsHeaders.getPartitionId());
        doc.setSecretType(secret.getSecretType());
        doc.setSecretValue(cryptographyUtil.encryptData(secret.toString()));

        cosmosStore.upsertItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getSubscriptionContainerName(), dpsHeaders.getPartitionId(), doc);
        return true;
    }
}
