// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.subscriber.pubsub;

import com.microsoft.azure.servicebus.management.ManagementClient;
import com.microsoft.azure.servicebus.primitives.MessagingEntityAlreadyExistsException;
import com.microsoft.azure.servicebus.primitives.MessagingEntityNotFoundException;
import org.opengroup.osdu.azure.serviceBusManager.IManagementClientFactory;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

@Component
@ConditionalOnProperty(value = "azure.serviceBus.enabled", havingValue = "true", matchIfMissing = true)
public class PullSubscription implements IPubsubSubscription {
    private static final String RESOURCE_PROVISIONING_ERROR_MESSAGE = "Resource Conflict Occurred. Another conflicting operation may be in progress";
    @Autowired
    private IManagementClientFactory factory;
    @Autowired
    private DpsHeaders dpsHeaders;
    @Autowired
    private JaxRsDpsLog logger;

    public void createSubscription(String subscriptionId, String topicAliasName) {
        ManagementClient managementClient = factory.getManager(dpsHeaders.getPartitionId());
        try {
            managementClient.createSubscription(topicAliasName, subscriptionId);
            logger.debug(String.format("Pull Subscription with id %s created successfully", subscriptionId));
        } catch (Exception e) {
            if (e.getMessage().contains(RESOURCE_PROVISIONING_ERROR_MESSAGE)) {
                logger.error("Another request is trying to create the same Pull subscription");
                throw new AppException(409, "Conflict", "Another request is trying to create the same Pull subscription");
            } else if (e instanceof MessagingEntityAlreadyExistsException) {
                logger.error(String.format("Pull Subscription with id %s already exist.", subscriptionId));
                throw new AppException(500, "Server Error", "Pull Subscription already exists");
            } else {
                logger.error("Creating Pull Subscription failed with error: " + e.toString());
                throw new AppException(500, "Server Error", "Unexpected error creating Pull subscription");
            }
        }
    }

    public void deleteSubscription(String subscriptionId, String topicAliasName) {
        ManagementClient managementClient = factory.getManager(dpsHeaders.getPartitionId());
        try {
            managementClient.deleteSubscription(topicAliasName, subscriptionId);
            logger.debug(String.format("Pull Subscription with id %s deleted successfully", subscriptionId));
        } catch (Exception e) {
            if (e.getMessage().contains(RESOURCE_PROVISIONING_ERROR_MESSAGE)) {
                logger.error("Another request is trying to delete the same Pull subscription");
                throw new AppException(409, "Conflict", "Another request is trying to delete the same Pull subscription");
            } else if (e instanceof MessagingEntityNotFoundException) {
                logger.error(String.format("Pull Subscription with id %s does not exist.", subscriptionId));
                throw new AppException(404, "Not found", String.format("Pull Subscription with id %s does not exist.", subscriptionId));
            } else {
                logger.error("Deleting Pull Subscription failed with error: " + e.toString());
                throw new AppException(500, "Server Error", "Unexpected error deleting Pull subscription");
            }
        }
    }

    public void checkIfSubscriptionExists(String subscriptionId, String topicAliasName) {
        ManagementClient managementClient = factory.getManager(dpsHeaders.getPartitionId());
        try {
            managementClient.getSubscription(topicAliasName, subscriptionId);
            logger.debug(String.format("Pull Subscription with id %s fetched successfully", subscriptionId));
        } catch (Exception e) {
            if (e instanceof MessagingEntityNotFoundException) {
                logger.error(String.format("Pull Subscription with id %s does not exist.", subscriptionId));
                throw new AppException(404, "Not found", String.format("Pull Subscription with id %s does not exist.", subscriptionId));
            } else {
                logger.error("Fetching Pull subscription failed with error: " + e.toString());
                throw new AppException(500, "Server Error", "Unexpected error while Fetching Pull subscription");
            }
        }
    }
}
