/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.provider.gcp.subscriber;

import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class SubscriptionRepositoryTest {
    @Mock
    private IDatastoreAccess datastoreAccess;

    @Mock
    private ArrayList<Subscription> mockArrayList;

    @InjectMocks
    private SubscriptionRepository sut;

    @Test
    public void should_throw429_when_subscriptionQuotaLimitReached() throws Exception {
        Subscription s = new Subscription();
        when(mockArrayList.size()).thenReturn(50);
        when(datastoreAccess.getAll()).thenReturn(mockArrayList);

        try {
            sut.create(s);
            fail("Expected error");
        } catch (AppException e) {
            assertEquals(429, e.getError().getCode());
            assertTrue(e.getError().getMessage().startsWith("Number of subscription allowed have already reached to maximum"));
            assertEquals("Subscription quota reached", e.getError().getReason());
        }
    }

    @Test
    public void should_return_subscription_no_error() throws Exception {
        Subscription s = new Subscription();
        when(mockArrayList.size()).thenReturn(0);
        when(datastoreAccess.getAll()).thenReturn(mockArrayList);
        when(datastoreAccess.create(s)).thenReturn(s);

        Subscription output = sut.create(s);
        assertEquals(output, s);
    }

    @Test
    public void should_get_from_dataStore() {
        Subscription s = new Subscription();
        when(datastoreAccess.get("id")).thenReturn(s);
        Subscription output = sut.get("id");
        assertEquals(output, s);
    }

    @Test
    public void should_query_from_dataStore() {
        List<Subscription> subs = new ArrayList<>();
        Subscription s = new Subscription();
        subs.add(s);
        when(datastoreAccess.query("id")).thenReturn(subs);
        List<Subscription> output = sut.query("id");
        assertEquals(s, output.get(0));
    }

    @Test
    public void should_delete_from_cache_and_datastore() {
        Subscription s = new Subscription();
        when(datastoreAccess.delete("id")).thenReturn(true);
        sut.delete("id");
    }

    @Test
    public void should_patch_secret() {
        Subscription s = new Subscription();
        s.setId("id");
        HmacSecret secret = new HmacSecret();
        secret.setValue("newValue");
        when(datastoreAccess.patch("id", secret)).thenReturn(true);
        assertTrue(sut.patch(s, secret));
    }

    @Test
    public void should_return_if_exception_in_datastore(){
        Subscription s = new Subscription();
        HmacSecret secret = new HmacSecret();
        secret.setValue("newValue");
        assertFalse(sut.patch(s, secret));
    }

    @Test
    public void should_get_all() throws Exception {
        Subscription s = new Subscription();
        ArrayList<Subscription> subs = new ArrayList<>();
        subs.add(s);
        when(datastoreAccess.getAll()).thenReturn(subs);
        List<Subscription> list = sut.getAll();
        assertEquals(subs.get(0), list.get(0));
    }
}