/*
  Copyright 2021 Google LLC
  Copyright 2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package org.opengroup.osdu.register.provider.reference.subscriber;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

import java.util.Collections;
import java.util.Optional;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.provider.reference.mapper.SubscriptionMapper;
import org.opengroup.osdu.register.provider.reference.model.document.SubscriptionDocument;
import org.opengroup.osdu.register.provider.reference.repository.mongo.SubscriptionDocumentRepository;
import org.opengroup.osdu.register.subscriber.model.Subscription;

@RunWith(MockitoJUnitRunner.class)
public class ReferenceSubscriptionRepositoryTest {

  @Mock
  private SubscriptionDocumentRepository subscriptionDocumentRepository;

  @Mock
  private SubscriptionMapper subscriptionMapper;

  @InjectMocks
  private ReferenceSubscriptionRepository referenceSubscriptionRepository;

  private Subscription subscription;
  private SubscriptionDocument subscriptionDocument;

  @Before
  public void setUp() {
    subscription = new Subscription();
    subscription.setId("subscriptionId");

    subscriptionDocument = new SubscriptionDocument();
    subscriptionDocument.setId("documentId");
  }

  @Test
  public void should_createSubscription_when_subscriptionDoesNotExist() throws Exception {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(false);
    when(subscriptionMapper.toDocument(subscription)).thenReturn(subscriptionDocument);
    when(subscriptionDocumentRepository.save(subscriptionDocument)).thenReturn(
        subscriptionDocument);
    when(subscriptionMapper.toSubscription(subscriptionDocument)).thenReturn(subscription);

    assertEquals(subscription, referenceSubscriptionRepository.create(subscription));
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_creatingSubscriptionWhichExists() throws Exception {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(true);

    referenceSubscriptionRepository.create(subscription);
  }

  @Test
  public void should_getSubscription_when_subscriptionExists() {
    when(subscriptionDocumentRepository.findById(any())).thenReturn(
        Optional.of(subscriptionDocument));
    when(subscriptionMapper.toSubscription(subscriptionDocument)).thenReturn(subscription);

    assertEquals(subscription, referenceSubscriptionRepository.get(any()));
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_gettingSubscriptionWhichDoesNotExist() {
    referenceSubscriptionRepository.get(any());
  }

  @Test
  public void should_updateSubscription_when_subscriptionExists() {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(true);
    when(subscriptionMapper.toUpdatedDocument(subscription, null)).thenReturn(
        subscriptionDocument);

    assertEquals(true, referenceSubscriptionRepository.patch(subscription, any()));
  }

  @Test
  public void shouldNot_updateSubscription_when_subscriptionDoesNotExist() {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(false);

    assertEquals(false, referenceSubscriptionRepository.patch(subscription, any()));
  }

  @Test
  public void should_deleteSubscription_when_subscriptionExists() {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(true);

    assertEquals(true, referenceSubscriptionRepository.delete(any()));
  }

  @Test
  public void shouldNot_deleteSubscription_when_subscriptionDoesNotExist() {
    when(subscriptionDocumentRepository.existsById(any())).thenReturn(false);

    assertEquals(false, referenceSubscriptionRepository.delete(any()));
  }

  @Test
  public void should_querySubscriptionByNotificationId() {
    when(subscriptionDocumentRepository.findByNotificationId(any())).thenReturn(
        Collections.singletonList(subscriptionDocument));
    when(subscriptionMapper.toSubscription(any())).thenReturn(subscription);

    assertEquals(Collections.singletonList(subscription),
        referenceSubscriptionRepository.query(any()));
  }

  @Test
  public void should_getAllSubscriptions() throws Exception {
    when(subscriptionDocumentRepository.findAll()).thenReturn(
        Collections.singletonList(subscriptionDocument));
    when(subscriptionMapper.toSubscription(any())).thenReturn(subscription);

    assertEquals(Collections.singletonList(subscription), referenceSubscriptionRepository.getAll());
  }
}
